Scriptname FPV_BadEndScript extends Quest

Actor Property PlayerRef Auto Const Mandatory
Faction Property RaiderFaction Auto Const Mandatory
Faction Property GunnerFaction Auto Const Mandatory
Faction Property SuperMutantFaction Auto Const Mandatory
Faction Property SynthFaction Auto Const Mandatory
Faction Property InstituteFaction Auto Const Mandatory
GlobalVariable Property AFV_Global_BadEndFatal Auto Const Mandatory
GlobalVariable Property AFV_Global_Debug Auto Const
ReferenceAlias Property aggressor Auto
Scene Property FPV_BadEndScene Auto

Bool FatalBadEnd
Furniture[] BP_Furniture
AAF:AAF_API AAF_API
ScriptObject AFVHellPlayerScript
Import FPV_Logger

Event OnStageSet(int stage, int stageItem)
	If (stage == 0)
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV BadEnd: Quest started")
		LoadAAF()
		LoadFurniture()
	ElseIf (stage == 10)
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV BadEnd: Starting the main scene")
		FPV_BadEndScene.Start()
	ElseIf (stage == 20)
		ExecutePlayer()
	ElseIf (stage == 255)
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV BadEnd: Stopping quest")
		SafeStop()
	EndIf
EndEvent

Bool Function SetFatality()
	Actor Rapist = Aggressor.GetReference() as Actor
	FatalBadEnd = ((AFV_Global_BadEndFatal.GetValueInt() == 2) || ((AFV_Global_BadEndFatal.GetValueInt() == 1) && (Rapist.GetLevel() > (PlayerRef.GetLevel() + 9))))
	AFVTrace(AFV_Global_Debug.GetValue(),"AFV BadEnd: FatalBadEnd " + FatalBadEnd + " (" + AFV_Global_BadEndFatal.GetValueInt() + ")")
	Return FatalBadEnd
EndFunction

; V1.0 Completely rewritten to allow random endings within a range
; This relies on the AAF_Violate_BadEnd_TagData.xml
; Raiders and Super Mutants can impale if Bad End is fatal
; Synths get to use Institute Chair and don't hang or impale victims
; Super Mutants too stupid to electrocute so no chairs
Function ExecutePlayer()
	Actor Rapist = Aggressor.GetReference() as Actor
	Actor[] actors = new Actor[1]
	Int minIndex = 1
	Int maxIndex = 3
	Int furnIndex
	String metaString = "BADEND"
	
	; Fade to black.  We will fade back in during the OnSceneInit event
	Game.FadeOutGame(True, True, 0.0, 2.0, True)

	If (FatalBadEnd)
		metaString = "BADENDFATAL"
		If Rapist.IsInFaction(SuperMutantFaction) || Rapist.IsInFaction(RaiderFaction)
			maxIndex += 1
		EndIf
	EndIf
	
	If Rapist.IsInFaction(InstituteFaction) || Rapist.IsInFaction(SynthFaction)
		minIndex = 0
		maxIndex = 1
	ElseIf Rapist.IsInFaction(SuperMutantFaction)
		minIndex = 2
	EndIf
	
	furnIndex = Utility.RandomInt(MinIndex,MaxIndex)
	
	; Handcuffs or DD ropes if needed
	If (furnIndex == 2 || furnIndex == 3)
		TiePlayersHands()
	EndIf
	
	; now miracle the device into place, orient it, and start the animation
	ObjectReference FurnRef = PlayerRef.PlaceAtMe(BP_Furniture[furnIndex])
	FurnRef.MoveToNearestNavmeshLocation()
	FurnRef.SetAngle(0.0, 0.0, PlayerRef.GetAngleZ())
	
	AAF:AAF_API:SceneSettings settings = AAF_API.GetSceneSettings()
	If PlayerLegsRestrained()
		settings.excludeTags="LegsFree"
	Else
		settings.excludeTags="LegsTied"
	EndIf
	If (furnIndex > 3)
		settings.duration = 94
	Else
		settings.duration = 98
	EndIf
	settings.includeTags = "Sequential"
	settings.locationObject = FurnRef
	settings.meta = metaString
	actors[0] = PlayerRef
	AAF_API.StartScene(actors, settings)
EndFunction

Function LoadFurniture()
	BP_Furniture = new Furniture[5]
	BP_Furniture[0] = Game.GetFormFromFile(0x0000C002, "AAF_BadEnd_Animations.esp") as Furniture ; Institute chair
	BP_Furniture[1] = Game.GetFormFromFile(0x0000C000, "AAF_BadEnd_Animations.esp") as Furniture ; Makeshift chair
	BP_Furniture[2] = Game.GetFormFromFile(0x0000C006, "AAF_BadEnd_Animations.esp") as Furniture ; Strangling Pole
	BP_Furniture[3] = Game.GetFormFromFile(0x0000C004, "AAF_BadEnd_Animations.esp") as Furniture ; Gallows
	BP_Furniture[4] = Game.GetFormFromFile(0x0000C005, "AAF_BadEnd_Animations.esp") as Furniture ; Impaling Pole
EndFunction

Function LoadAAF()
	AAF_API = Game.GetFormFromFile(0x00000F99, "AAF.esm") as AAF:AAF_API
	RegisterForCustomEvent(AAF_API, "OnSceneInit")
	RegisterForCustomEvent(AAF_API, "OnAnimationStart")
	RegisterForCustomEvent(AAF_API, "OnAnimationStop")
EndFunction

bool Function PlayerLegsRestrained()
	If Game.IsPluginInstalled("Devious Devices.esm")
		Keyword DD_kw_EffectType_RestrainedLegs = Game.GetFormFromFile(0x000A2F15, "Devious Devices.esm") as Keyword
		If PlayerRef.WornHasKeyword(DD_kw_EffectType_RestrainedLegs)
			AFVTrace(AFV_Global_Debug.GetValue(),"AFV BadEnd: Player's legs are restrained")
			Return True
		Endif
	Endif
	AFVTrace(AFV_Global_Debug.GetValue(),"AFV BadEnd: Player's legs are free")
	Return False
EndFunction

; Check if the player's hands are tied; if not, tie them with simple DD wrist ropes
Function TiePlayersHands()
	If Game.IsPluginInstalled("RealHandcuffs.esp")
		Keyword RH_ma_handcuffs = Game.GetFormFromFile(0x0000082A, "RealHandcuffs.esp") as Keyword
		If PlayerRef.WornHasKeyword(RH_ma_handcuffs)
			AFVTrace(AFV_Global_Debug.GetValue(),"AFV BadEnd: Player is handcuffed")
			Return
		EndIf
	EndIf
	If Game.IsPluginInstalled("Devious Devices.esm")
		Keyword DD_kw_ItemType_WristCuffs = Game.GetFormFromFile(0x0001196C, "Devious Devices.esm") as Keyword
		If PlayerRef.WornHasKeyword(DD_kw_ItemType_WristCuffs)
			AFVTrace(AFV_Global_Debug.GetValue(),"AFV BadEnd: Player's hands are restrained")
		Else
			Quest DDQuest = Game.GetFormFromFile(0x09004C50, "Devious Devices.esm") as Quest
			ScriptObject DDlibs = DDQuest.CastAs("DD:DD_Library")
			Armor DD_Wristcuffs_Rope_Inventory = Game.GetFormFromFile(0x001455B6, "Devious Devices.esm") as Armor
			
			var[] DDVar = new var[6]
			DDVar[0] = PlayerRef
			DDVar[1] = DD_Wristcuffs_Rope_Inventory
			DDVar[2] = none
			DDVar[3] = true
			DDVar[4] = none
			DDVar[5] = "none"
			bool bResult = DDLibs.CallFunction("EquipDevice", DDVar) as bool
			AFVTrace(AFV_Global_Debug.GetValue(),"AFV BadEnd: DD EquipDevice returned " + bResult)
		Endif
	Else
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV BadEnd: DD not installed, can't tie player's hands")
	Endif
EndFunction

Function SafeStop()
	Aggressor.Clear()
	Stop()
EndFunction

Bool Function CheckPurgatory()
	If (Game.IsPluginInstalled("AFVPurgatory.esp"))
		Quest AFVHellPlayerQuest = Game.GetFormFromFile(0x00EC25, "AFVPurgatory.esp") as Quest
		AFVHellPlayerScript = AFVHellPlayerQuest.CastAs("AFVHellPlayerQuestScript")
		Return AFVHellPlayerScript.GetPropertyValue("ModEnabled") as Bool
	EndIf
	Return False
EndFunction
			
Event AAF:AAF_API.OnSceneInit(AAF:AAF_API akSender, Var[] akArgs)
	int iResult =  akArgs[0] as int
	If iResult > 0
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV BadEnd: SceneInit event: error [" + iResult + "]")
		AFVNotify(True, "AAF OnSceneInit status [" + iResult + "]")
		Game.FadeOutGame(false, true, 0.0, 0.0, false)
		SetStage(255)
	Else
		String sMeta = akArgs[5] as String
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV BadEnd: SceneInit meta: " + sMeta)
		
		If (LL_FourPlay.StringSubstring(sMeta, 0, 6) == "BADEND")
			Game.FadeOutGame(false, true, 3.0, 2.0, false)
		Endif
	Endif
EndEvent

Event AAF:AAF_API.OnAnimationStart(AAF:AAF_API akSender, Var[] akArgs)
	Int iResult =  akArgs[0] as Int
	If iResult > 0
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV BadEnd: OnAnimationStart event: error [" + iResult + "]")
		AFVNotify(True, "AAF OnAnimationStart status [" + iResult + "]")
		Game.FadeOutGame(false, true, 0.0, 0.0, false)
		SetStage(255)
	Else
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV BadEnd: AnimationStart event")
	EndIf
EndEvent

Event AAF:AAF_API.OnAnimationStop(AAF:AAF_API akSender, Var[] akArgs)
	int iResult =  akArgs[0] as int
	If iResult > 0
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV BadEnd: OnAnimationStop event: error [" + iResult + "]")
		AFVNotify(True, "AAF OnAnimationStop status [" + iResult + "]")
		SetStage(255)
	Else
		String sPosition = akArgs[2] as String
		String sMeta = akArgs[4] as String
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV BadEnd: AnimationStop position: " + sPosition + " meta: " + sMeta)
		
		If (sMeta == "BADEND")
			SetStage(255)
		Elseif (sMeta == "BADENDFATAL")
			If CheckPurgatory()
				AFVHellPlayerScript.CallFunction("Startup", new Var[0])
			Else
				PlayerRef.KillEssential()
			EndIf
			Setstage(255)
		Endif
	Endif
EndEvent
