Scriptname FPV_RSEAbductionScript extends Quest Conditional

Faction Property RaiderFaction Auto
Faction Property GunnerFaction Auto
Faction Property SuperMutantFaction Auto
GlobalVariable Property AFV_Global_Debug Auto
Keyword Property ActorTypeHuman Auto Const
Keyword Property ActorTypeGhoul Auto Const
Keyword Property ActorTypeSuperMutant Auto Const
ReferenceAlias Property Captor Auto
Scene Property FPV_RSEAbductionStartDialogue Auto

GlobalVariable FactionDungeon
Quest RSEAbductionLocation
Quest RSEAbductionMain
Quest RSEII_Mod_CSA_Surrender_Companion
Quest RSEII_Mod_CSA_Abduction_Function_Companions
Import FPV_Logger

Bool Function LoadRSEAbductions()
	If Game.IsPluginInstalled("RSE_II_CSA.esp")
		FactionDungeon = Game.GetFormFromFile(0x0000B7DF,"RSE_II_CSA.esp") as GlobalVariable
		RSEAbductionLocation = Game.GetFormFromFile(0x0000B7DE,"RSE_II_CSA.esp") as Quest
		RSEAbductionMain = Game.GetFormFromFile(0x0000B7DB,"RSE_II_CSA.esp") as Quest
		RSEII_Mod_CSA_Surrender_Companion = Game.GetFormFromFile(0x00000FA0,"RSE_II_CSA.esp") as Quest
		RSEII_Mod_CSA_Abduction_Function_Companions = Game.GetFormFromFile(0x0000F510,"RSE_II_CSA.esp") as Quest
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RSEAbductionScript: RSE II CSA Loaded")
		Return True
	Else
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RSEAbductionScript: RSE II CSA not found")
		Return False
	Endif
EndFunction

Bool Function ValidateCaptor(Actor akAggressor)
	Return akAggressor.HasKeyword(ActorTypeHuman) || akAggressor.HasKeyword(ActorTypeGhoul) || akAggressor.HasKeyword(ActorTypeSuperMutant)
EndFunction

Function AbductPlayer(Actor akAggressor)
	; To send the player to an abduction site, check the last attacker / rapist against a faction
	; and set the FactionDungeon flag variable to indicate which dungeon type to lock in.
	If akAggressor.IsInfaction(RaiderFaction)
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RSEAbductionScript: Raider abduction")
		FactionDungeon.SetValueInt(1)
	Elseif akAggressor.IsInFaction(GunnerFaction)
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RSEAbductionScript: Gunner abduction")
		FactionDungeon.SetValueInt(2)
	Elseif akAggressor.IsInFaction(SuperMutantFaction)
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RSEAbductionScript: Super Mutant abduction")
		FactionDungeon.SetValueInt(3)
	Elseif akAggressor.HasKeyword(ActorTypeHuman) || akAggressor.HasKeyword(ActorTypeGhoul)
		FactionDungeon.SetValueInt(Utility.RandomInt(1,3))
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RSEAbductionScript: Random abduction [" + FactionDungeon.GetValueInt() + "]")
	Else
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RSEAbductionScript: Captor not valid")
		Return
	Endif

	; Now we start the quest to find the location
	RSEAbductionLocation.Stop()
	RSEAbductionLocation.Start()
	
	LocationAlias PlayerCaptureLoc = (RSEAbductionLocation.CastAs("RSEII_CSA:RSEII_CSAAbductionDebugScript")).GetPropertyValue("PlayerCaptureLoc") as LocationAlias
	If PlayerCaptureLoc.GetLocation()
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RSEAbductionScript: loc found: " + PlayerCaptureLoc.GetLocation().GetName())
		Captor.ForceREFto(akAggressor)
		FPV_RSEAbductionStartDialogue.Start()	
		While (FPV_RSEAbductionStartDialogue.IsPlaying())
			Utility.Wait(0.5)
		EndWhile
		Captor.Clear()
		
		If RSEAbductionLocation.IsRunning() && PlayerCaptureLoc.GetLocation()
			; first start the main CSA companion finder quest
			RSEII_Mod_CSA_Surrender_Companion.Stop()
			RSEII_Mod_CSA_Surrender_Companion.Start()
	
			; then the abduction-specific companion finder, which pulls from the one above
			RSEII_Mod_CSA_Abduction_Function_Companions.Stop()
			RSEII_Mod_CSA_Abduction_Function_Companions.Start()
		
			; finally start the whole abduction shebang, which pulls companion and location aliases from the above three quests
			RSEAbductionMain.Reset()
			RSEAbductionMain.Start()
			 
			AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RSEAbductionScript: RSE Abduction quest started")
			
			; Stop the main CSA companion finder quest since its packages mess with the abduction
			Utility.Wait(5.0)
			RSEII_Mod_CSA_Surrender_Companion.Stop()
		Else
			AFVTrace(AFV_Global_Debug.GetValue(),"RSEAbductionLocation quest is not running, aborting")
		EndIf
	Else
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RSEAbductionScript: RSE Abduction Location not found, aborting")
	Endif
EndFunction
