Scriptname FPV_RaiderPetScript extends Quest Conditional

; Interface script for Violate to start the Raider Pet quest
; Based on Stobor's extremely helpful Example Quest Script
;
; FPV_RaiderPet starts enabled, starts up the Raider Pet quest if
; it is not already running, and then waits at stage 0.  Note that this
; bypasses the need to go to a settlement or do the other Raider Pet
; startup stuff.
;
; When a Raider Pet ending may be used, FPV_OnHit populates the Captor 
; refAlias and calls the CheckLocationAndActors() function.  If returns
; true, FPV_OnHit sets this quest's stage to 10 which actually captures
; the player and starts the enslavement dialogue.  After the dialogue, 
; this quest goes to Stage 50, where the alias is cleared, and waits.

Actor Property PlayerRef Auto Const
GlobalVariable Property AFV_Global_Debug Auto Const
Faction Property CaptiveFaction Auto Const
Faction Property RaiderFaction Auto Const
Location Property PreviousLocation Auto
LocationRefType Property LocRefTypeBoss Auto Const
Quest Property FPV_Follower Auto
Quest Property FPV_Spectator Auto
ReferenceAlias Property Captor Auto
Scene Property FPV_RaiderPetStartDialogue Auto Const

ScriptObject RaiderPetScr = None
Quest RaiderPetQuest = None

Import FPV_Logger

Function Setup()
	;; Placeholder
EndFunction

Function Cleanup()
	Captor.Clear()
EndFunction

Function Maintenance()
	;; Check for external dependencies:
	If (Game.isPluginInstalled("Raider Pet.esp"))
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RaiderPet: Raider Pet mod found")
		RaiderPetQuest = Game.GetFormFromFile(0x00000F99, "Raider Pet.esp") as Quest
		RaiderPetScr = (RaiderPetQuest).CastAs("RaiderPet:DialogueQuestScr")
		
		If !RaiderPetQuest.isRunning()
			RaiderPetQuest.Stop()
			RaiderPetQuest.Start()
			Utility.Wait(0.1)
		EndIf
	Else
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RaiderPet: Raider Pet mod not found")
		RaiderPetScr = None
	EndIf

	;; Ensure that events are registered:
	RegisterForRemoteEvent(PlayerRef, "OnPlayerLoadGame")
EndFunction

Event OnQuestInit()
    Setup()
    Maintenance()
EndEvent

Event OnQuestShutdown()
	Cleanup()
EndEvent

Event Actor.OnPlayerLoadGame(Actor akSender)
    Maintenance()
EndEvent

Event OnStageSet(int stage, int stageItem)
	If (stage == 0)
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RaiderPet: Quest started")
		Setup()
		Maintenance()
	ElseIf (stage == 10)
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RaiderPet: Capturing Player")
		FPV_RaiderPetStartDialogue.Start()	
		While (FPV_RaiderPetStartDialogue.IsPlaying())
			Utility.Wait(0.1)
		EndWhile
	ElseIf (stage == 50)
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RaiderPet: Setup complete")
		Cleanup()
	ElseIf (stage == 255)
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RaiderPet: Stopping quest")
		Cleanup()
		Stop()
	EndIf
EndEvent

; Ensure the captor is a Raider, the location has a Boss, and the player isn't already enslaved. 
; If the location isn't appropriate, e.g. a settlement or owned by someone other than Raiders, 
; the player will have to deal with it themselves by escaping. To prevent loops, this function 
; will not allow the player to be enslaved consecutively in the same location.
Bool Function CheckLocationAndActors()
	Actor CaptorActor = Captor.GetReference() as Actor
	If RaiderPetQuest != None && RaiderPetQuest.IsRunning()
		If !PlayerRef.IsInFaction(CaptiveFaction)
			If PlayerRef.GetCurrentLocation() != PreviousLocation
				If PlayerRef.GetCurrentLocation().HasRefType(LocRefTypeBoss)
					If CaptorActor.IsInFaction(RaiderFaction) 
						If CaptorActor.isDead() == false
							Return true
						Else
							AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RaiderPet: Captor is dead")
						EndIf
					Else
						AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RaiderPet: Captor is not a Raider")
					EndIf
				Else
					AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RaiderPet: No boss in Location")
				EndIf
			Else
				AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RaiderPet: Player was just captive in this Location")
			EndIf
		Else
			AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RaiderPet: Player is already a captive")
		EndIf
	Else
		AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RaiderPet: Raider Pet quest not running")
	EndIf
	Return False
EndFunction
;;
;; This is an example of using a remote function to notify the Raider Pet mod
;; that the player character should be treated as a potential captive.  
;;
Function PrepareForCaptivity()
	;; Do any desired preparations, like placing all weapons in a container...
	AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RaiderPet: PrepareForCaptivity")
	;; Tell Raider Pet to treat the character as a potential captive:
	If ( RaiderPetScr != None )
		RaiderPetScr.CallFunction( "MakeCaptive", new Var[0] )
	EndIf

	;; At this point, the character is in CaptiveFaction but not yet claimed 
	;; by a specific raider gang.  The dialogue option to be claimed is now
	;; active.  When and If a raider takes that step, the TakenCaptive event
	;; will be sent.  If the character has picked up (but not equipped) a
	;; weapon prior to that, the weapons will be confiscated at that point.

	;; If the character picks up and EQUIPS a weapon prior to being claimed,
	;; they will revert to normal behavior without ever generating either the
	;; TakenCaptive or Escaped events.

EndFunction

;;
;; This is an example of using a remote function to notify the Raider Pet mod
;; that the player character has been captured by this specific Actor.  It
;; is recommended to let this happen naturally via the Raider Pet dialogue, 
;; instead of forcing it like this... but If you really want full control of
;; that process, this approach will also work.
;;
;; This call must be made with the player character IN THE SAME LOCATION as 
;; the Actor in question, the cell must be loaded, and the Actor must be in 
;; the Enabled state.  Otherwise quest aliases will not fill properly.
;;
Function StakeClaim( Actor akSpeaker )
	;; The 'MakeCaptive' call should already have been made, or the results
	;; will be unpredictable.  And probably horrible.
	AFVTrace(AFV_Global_Debug.GetValue(),"AFV FPV_RaiderPet: StakeClaim " + akSpeaker.GetLeveledActorBase().GetName())
	;; Tell Raider Pet to treat the character a captive of this Actor:
	If ( RaiderPetScr != None )
		Var[] kArgs = new Var[1]
		kArgs[0] = akSpeaker
		RaiderPetScr.CallFunction( "ClaimOwnership", kArgs )
		PreviousLocation = PlayerRef.GetCurrentLocation()
	EndIf

	;; The 'TakenCaptive' event will be sent by the function call above.

EndFunction

;;
;; This is an example of using a remote function to notify the Raider Pet mod
;; that the player character has made a break for freedom, causing hostilities
;; to resume.  So If you wanted to add additional conditions that could cause
;; that to occur (e.g. breaking into the boss safe), you would use this call.
;;
Function BreakLoose()
	;; The 'MakeCaptive' call should already have been made, or the results
	;; will be unpredictable.  The 'ClaimOwnership' call does NOT necessarily
	;; have to precede this.

	;; Tell Raider Pet to resume hostilities on sight:
	If ( RaiderPetScr != None )
		RaiderPetScr.CallFunction( "BreakForFreedom", new Var[0] )
	EndIf

    ;; The 'Escaped' event will be sent by the function call above, If there
    ;; has been a prior 'ClaimOwnership' call.

EndFunction

